package produttore_consumatore;

/**
 * Oggetto che recupera i dati prodotti dal Produttore.
 *
 * @author <a href="mailto:ervincio@tiscali.it">Vincenzo Manzoni</a>
 */

public class Consumatore extends Thread {

        /** Variabile per la condivisione dei dati Produttori - Consumatori. */
        private Buffer data;

        /** Nome identificativo del processo Produttore. */
        private String name;

        /**
         * Istanzia un Consumatore, attribuendogli un nome e una variabile
         * per lo scambio di dati con i produttori
         *
         * @param name Nome identificativo del Consumatore.
         *             Dovrebbe essere nelle forma CONSUMATOREn, dove
         *             n rappresenta un numero naturale.
         * @param data Variabile, di tipo Buffer, per lo scambio di dati con i
         *             produttori.
         */
        public Consumatore(String name, Buffer data) {
                this.data = data;
                this.name = name;
        }

        /**
         * Recupera dal buffer gli oggetti di tipo Integer che i produttori hanno
         * piazzato dentro.
         * Prima di recuperarli verifica che la variabile di scambio non sia vuota.
         * Se lo  si mette in <code>wait()</code>.
         */
        public void run() {

                // Variabile in cui si mettere gli oggetti di tipo Integer recuperati
                Integer i = null;

                // Ciclo infinito
                while(true) {
                        try {

                                // Lock sulla variabile
                                synchronized(data) {

                                        // Se il buffer  vuoto vado in wait()
                                        if(data.isEmpty())
                                                data.wait();
                                        // Altrimenti recupero l'oggetto e lo stampo a video
                                        else {
                                                i = (Integer) data.remove();
                                                System.out.println("[" + name + "]: Ho estratto dal " +
                                                        "buffer " + i.intValue());
                                        }
                                }

                                /*
                                 * N.B.: In questo caso non uso lo Sleep, perch se il dato
                                 * non  pronto vado in wait() e mi risveglio non appena il
                                 * produttore mi manda un messaggio di notify()
                                 */
                        }
                        catch (InterruptedException ie) {
                        }
                }
        }
}
